/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "slider.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define SLIDER_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void slider_cfg_setup ( slider_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->oe = HAL_PIN_NC;
    cfg->csl = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

SLIDER_RETVAL slider_init ( slider_t *ctx, slider_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;
    
    digital_in_init( &ctx->miso, cfg->miso );

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return SLIDER_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, SLIDER_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->oe, cfg->oe );
    digital_out_init( &ctx->csl, cfg->csl );

    return SLIDER_OK;

}

void slider_default_cfg ( slider_t *ctx )
{
    slider_enable_led_output( ctx, 0 );
}

void slider_generic_transfer ( slider_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    Delay_us(22);
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );
    Delay_us(22);
}

uint32_t slider_read_adc ( slider_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t buff_in[ 4 ];
    uint8_t buff_out[ 4 ];
    uint32_t ret_val;
    uint8_t i;

    block.tx_buffer = 0;
    block.tx_length = 0;
    block.rx_buffer = buff_out;
    block.rx_length = 4;

    slider_generic_transfer( ctx, &block );

    ret_val = buff_out[ 0 ];
    ret_val <<= 8;
    ret_val |= buff_out[ 1 ];
    ret_val <<= 8;
    ret_val |= buff_out[ 2 ];
    ret_val <<= 1;
    buff_out[ 3 ] >>= 7;
    ret_val |= buff_out[ 3 ];
    ret_val &= 0x003FFFFF;
    
    return ret_val;
}

uint32_t slider_read_adc_and_ready ( slider_t *ctx )
{
    uint32_t ret_val = 0;
    uint8_t check_flag = 1;
    
    while( check_flag )
    {
        spi_master_start( &ctx->spi );
        check_flag = digital_in_read( &ctx->miso );
        spi_master_stop( &ctx->spi );
    }
    
    ret_val = slider_read_adc( ctx );
    
    return ret_val;
}

void slider_enable_led_output ( slider_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->oe, state );
}

void slider_enable_output_laches ( slider_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->csl, state );
}

uint32_t slider_write_output ( slider_t *ctx )
{
    spi_master_transfer_data_t block;
    uint32_t adc_value = 0;
    uint8_t res = 0;
    uint16_t temp = 0;
    uint8_t cnt = 0;
    uint8_t output_buffer[ 2 ];
    uint8_t i;
    
    adc_value = slider_read_adc_and_ready( ctx );

    res = adc_value / 65457;
    
    for( cnt = 0; cnt < res; cnt++ )
    {
        if(cnt == 0)
        {
            temp |= 1;
        }else
        {
            temp = ( temp << 1 ) | 1;
        }
    }
    
    output_buffer[ 0 ] = temp >> 8;
    output_buffer[ 1 ] = temp;
    
    block.tx_buffer = output_buffer;
    block.tx_length = 2;
    block.rx_buffer = 0;
    block.rx_length = 0;

    slider_enable_output_laches( ctx, 1 );
    slider_generic_transfer( ctx, &block );
    slider_enable_output_laches( ctx, 0 );
    
    return adc_value;
}

// ------------------------------------------------------------------------- END

